/*************************************************************
*
* Description:
*	LPC236x - Example program using IAR
*
* Author[s]:
*	Frank Goetze
**************************************************************/
#include <iolpc2368.h>
#define XTALFREQ 12000000

#include "../../inc/pC_OS.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define	TASK_STK_SIZE         256
#define MEM_POOL_SIZE        1024

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];
OS_STK_TYPE Stk3[TASK_STK_SIZE];
U08 pool[MEM_POOL_SIZE];

U32 OS_SystemTicksCnt;

OS_MUX Mutex1;
OS_EVG Events1;
#if OS_TMR_EN
OS_SEM LedSem;
OS_TMR LedTimer;
OS_TMR ROnceTimer1;
OS_TMR ROnceTimer2;
#endif

void  Task1(void *Tdata);
void  Task2(void *Tdata);
void  Task3(void *Tdata);

int main(void)
{
//    MEMMAP = 2;                                            // on "SRAM_VIA_JLINK"
    
    OS_Init();
    OS_MemoryInit((OS_MEM OS_HUGE *)(&pool[0]), MEM_POOL_SIZE);
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], &Stk1, 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], &Stk2, 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], &Stk3, 12);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 11);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 13);
    OS_TaskCreate(Task3, NULL, (void *)&Stk3[TASK_STK_SIZE], 12);
#endif
    OS_MutexCreate(&Mutex1, 10);
    OS_EvgInit(&Events1);
#if OS_TMR_EN
    OS_SemInit(&LedSem, 0);
#endif
    
    OS_Start();
    
    while(1);
}

void OSTickISR(void)                                       // typical LPC236x pC/OS ISR
{
    T0IR = 1;
    
#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
}

void AppTickInit(void)
{                                                          // Configure the system tick
    U32  clk_freq;

    OS_SystemTicksCnt = 0;
    clk_freq  = XTALFREQ;                                  // CLK must be the main-oscillator here
    if(PLLSTAT & 0x03000000U) {                            // if PLL active
        while(!(PLLSTAT & 0x04000000U));                   // take care PLL is locked
        clk_freq /= (((PLLSTAT & 0x00FF0000U) >> 16) + 1); // get NSEL (dividier)
        clk_freq *= (2 * ((PLLSTAT & 0x00007FFFU) + 1));   // get MSEL (multiplier)
    }
    clk_freq /= ((CCLKCFG & 0x000000FFU) + 1);             // get CCLKSEL (dividier)
    switch((U32)(PCLKSEL0 & 0x0000000CU) >> 2) {           // get PCLK_TIM0 (dividier)
        case 0:  clk_freq = clk_freq / 4; break;
        case 1:  clk_freq = clk_freq;     break;
        case 2:  clk_freq = clk_freq / 2; break;
        default: clk_freq = clk_freq / 8; break;
    }
    clk_freq /= OS_TIMER_TICKS_PER_SEC;
    T0TCR          = 0;
    T0CCR          = 0;
    T0EMR          = 0;
    T0IR           = 0xFF;
    T0TC           = 0;
    T0PR           = 0;
    T0MR0          = clk_freq; 
    T0MCR          = 3;
    T0TCR          = 1;
    VICINTSELECT   = 0;
    VICINTENCLEAR  = (U32)(-1);
    VICPROTECTION  = 0;
    VICADDRESS     = 0;
    VICPROTECTION  = 0;
    VICINTSELECT  &= ~(1 << VIC_TIMER0); 
    VICVECTADDR4   = (U32)&OSTickISR;
    VICVECTPRIORITY4 = 15;
    VICINTENABLE  |= (1 << VIC_TIMER0);

}

void Task1(void *Tdata)
{
    U08 OS_HUGE *ptr;
    U08 returnOk;
    U32 event, i;

    if(Tdata != NULL) while(1);
    AppTickInit();
    ptr=NULL;
    returnOk=OS_MemAlloc((void OS_HUGE *)(&ptr), 30);      // memory-manager test
    returnOk=OS_MemFree((OS_MEM OS_HUGE *)ptr);
    while(1) {
        i=0;
        while(i<2500) i++;                                 // no Task-switch to MUTEX
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // first ok. - secound suspending
        if(!returnOk) {
            OS_TimeDly(5);
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            event = 0x10;
            returnOk=OS_EvgPend(&Events1, &event, OS_EVG_OR_C, OS_SUSPEND);
        }
    }
}

void Task2(void *Tdata)
{
    U32 i;
    U08 returnOk;

    Tdata = Tdata;
    while(1) {
        returnOk=OS_MutexPend(&Mutex1, OS_SUSPEND);        // suspending
        if(!returnOk) {
            i=0;
            while(i<2500) i++;                             // no Taskswitch to Task1 or Task3
            returnOk=OS_MutexPost(&Mutex1);
            OS_TimeDly(5);
            OS_EvgPost(&Events1, 0x10, OS_EVG_OR);
        }
    }
}

#if OS_TMR_EN
void ROnceTimer1CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStop(&LedTimer);                  // stope LED-timer
}

void ROnceTimer2CB(void *tdata)
{
    tdata = tdata;
    OS_TimerStart(&LedTimer, 0);              // restart LED-timer with org time
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerStart(&ROnceTimer2, 0);           // start both run-once (one auto-clear) timer again
}

void LedTimerCB(void *tdata)
{
    tdata = tdata;
    OS_SemPost(&LedSem);
}
#endif

void Task3(void *Tdata)
{
    U08 LED;
    
    Tdata = Tdata;                                         // Configure the LED on pin P1.18
 
    IO1DIR_bit.P1_18 = 1;
    IO1CLR_bit.P1_18 = 1;                                  // switch ON (control) 
#if OS_TMR_EN
    OS_TimerCreate(&ROnceTimer1, 500, ROnceTimer1CB, NULL, OS_TMR_ENABLE | OS_TMR_CLR);
    OS_TimerCreate(&ROnceTimer2, 1000, ROnceTimer2CB, NULL, OS_TMR_ENABLE | OS_TMR_RONCE);
    OS_TimerCreate(&LedTimer, 30, LedTimerCB, NULL, OS_TMR_ENABLE | OS_TMR_CYCL);
#endif    
    
    LED = 1;
    while(1) {
#if OS_TMR_EN
        OS_SemPend(&LedSem, OS_SUSPEND);                   // wait on timmer-triggert signal
#else
        OS_TimeDly(3);                                     // test timer-wakeup
#endif
        if(LED) IO1SET_bit.P1_18 = 1;
        else IO1CLR_bit.P1_18 = 1;
        LED = (LED)? 0:1;
    }
}
